//! Embassy DHCP Example
//!
//!
//! Set SSID and PASSWORD env variable before running this example.
//!
//! This gets an ip address via DHCP then performs an HTTP get request to some
//! "random" server
//!
//! Because of the huge task-arena size configured this won't work on ESP32-S2

//% FEATURES: embassy esp-wifi esp-wifi/wifi esp-hal/unstable
//% CHIPS: esp32 esp32s2 esp32s3 esp32c2 esp32c3 esp32c6

#![no_std]
#![no_main]

mod wifi;

use embassy_executor::Spawner;
use embassy_time::Delay;
use embassy_time::{Duration, Timer};
use embedded_hal_bus::spi::{ExclusiveDevice, NoDelay};
use esp_alloc as _;
use esp_backtrace as _;
use esp_hal::gpio::{Flex, Pin};
use esp_hal::time::Rate;
use esp_hal::{
    clock::CpuClock,
    spi::master::{Config, Spi},
    timer::timg::TimerGroup,
};
use esp_println::println;
use it8951::{interface::IT8951SPIInterface, IT8951};

esp_bootloader_esp_idf::esp_app_desc!();

#[esp_hal_embassy::main]
async fn main(spawner: Spawner) -> ! {
    esp_println::logger::init_logger_from_env();

    let config = esp_hal::Config::default().with_cpu_clock(CpuClock::max());
    let peripherals = esp_hal::init(config);
    let timg1 = TimerGroup::new(peripherals.TIMG1);

    esp_hal_embassy::init(timg1.timer0);

    esp_alloc::heap_allocator!(size: 72 * 1024); // why this value?

    let mut pin_ext_pwr = Flex::new(peripherals.GPIO5);
    let pin_busy = Flex::new(peripherals.GPIO27.degrade());
    let mut pin_reset = Flex::new(peripherals.GPIO23.degrade());
    let pin_chip_select = Flex::new(peripherals.GPIO15.degrade());
    let pin_miso = Flex::new(peripherals.GPIO13.degrade());
    let pin_mosi = Flex::new(peripherals.GPIO12.degrade());
    let pin_sck = Flex::new(peripherals.GPIO14.degrade());

    pin_ext_pwr.set_high();
    pin_reset.set_high();

    Timer::after(Duration::from_millis(1000)).await;

    let spi: Spi<'_, esp_hal::Blocking> = Spi::new(
        peripherals.SPI2,
        Config::default().with_frequency(Rate::from_mhz(10)),
    )
    .unwrap()
    .with_miso(pin_miso)
    .with_mosi(pin_mosi)
    .with_sck(pin_sck);

    let spi = ExclusiveDevice::new(spi, pin_chip_select, NoDelay).unwrap();

    let display_interface = IT8951SPIInterface::new(spi, pin_busy, pin_reset, Delay);
    // display_interface.set_busy_timeout(core::time::Duration::from_micros(500));

    let mut epd = match IT8951::new(display_interface, it8951::Config::default()).init(1605) {
        Ok(e) => e,
        Err(e) => panic!("{:?}", e),
    };

    epd.reset().unwrap();

    let stack = wifi::init(
        peripherals.TIMG0,
        peripherals.RNG,
        peripherals.RADIO_CLK,
        peripherals.WIFI,
        spawner,
    );

    println!("Waiting to get IP address...");
    loop {
        if stack.is_link_up() {
            if let Some(config) = stack.config_v4() {
                println!("Got IP: {}", config.address);
                break;
            }
        }
        Timer::after(Duration::from_millis(500)).await;
    }

    loop {
        Timer::after(Duration::from_millis(1_000)).await;
        // do nothing
    }

    // let mut rx_buffer = [0; 4096];
    // let mut tx_buffer = [0; 4096];
    // use core::net::Ipv4Addr;
    // use embassy_net::tcp::TcpSocket;
    // loop {
    //     Timer::after(Duration::from_millis(1_000)).await;

    //     let mut socket = TcpSocket::new(stack, &mut rx_buffer, &mut tx_buffer);

    //     socket.set_timeout(Some(embassy_time::Duration::from_secs(10)));

    //     let remote_endpoint = (Ipv4Addr::new(142, 250, 185, 115), 80);
    //     println!("connecting...");
    //     let r = socket.connect(remote_endpoint).await;
    //     if let Err(e) = r {
    //         println!("connect error: {:?}", e);
    //         continue;
    //     }
    //     println!("connected!");
    //     let mut buf = [0; 1024];
    //     loop {
    //         use embedded_io_async::Write;
    //         let r = socket
    //             .write_all(b"GET / HTTP/1.0\r\nHost: www.mobile-j.de\r\n\r\n")
    //             .await;
    //         if let Err(e) = r {
    //             println!("write error: {:?}", e);
    //             break;
    //         }
    //         let n = match socket.read(&mut buf).await {
    //             Ok(0) => {
    //                 println!("read EOF");
    //                 break;
    //             }
    //             Ok(n) => n,
    //             Err(e) => {
    //                 println!("read error: {:?}", e);
    //                 break;
    //             }
    //         };
    //         println!("{}", core::str::from_utf8(&buf[..n]).unwrap());
    //     }
    //     Timer::after(Duration::from_millis(30000)).await;
    // }
}
